#include "lcd_out.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "system.h"
#include "LCD.h"

// LCDの初期化
void lcd_init() {
	*lcd_on = 1;
	*lcd_blon = 1;
	LCD_Init();
}

// LCDのキャレットを１行目のはじめに移動する
void lcd_caret_reset() {
	LCD_Init();
}

// LCDのキャレットを２行目のはじめに移動する
void lcd_caret_reset2() {
	LCD_Line2();
}

// LCDに文字を表示する
void lcd_print(const char *str) {
	LCD_Show_Text(str);
}
